/* @(#)dem68030.dsc	6.8 2/9/93 11:18:07 */


/* Custom disassembler example for 68030 */


%% DEF
long	nopc_addr;
long	dest_address;
long	new_address;
int	nopc_part;
int	new_part;
int	pipeline;
int	datatr_opcpipe;
int	state;
int	part;
int	new_state;
int	last_opc;
int	last_opc_part;


#define	MAXPIPE		8

#define	FC_DATA		0b101
#define	FC_PROGRAM	0b110

%% EQU

/* lookup tables */
LT: STATUS_TAB, decode, ChkNextOpc
LT: Compute8bitBrAddr, FindDataRead, FindDataWrite
LT: CheckBrAddr, FindNextOpc, CompNewAddr
LT: FindAddrSeqBrk
LT: ChkAddrSeq
LT: SelUnused

/* index tables */
IT: print, ComputeReturnAddr
IT: FindNewOpcAddress

%% FORMAT

logo: "68030 C-DISA Example"

head: "68030 C-DISA Example"  33

Pods: threshold = {{TTL, TTL},
                   {TTL, TTL},
                   {TTL, TTL},
                   {TTL, TTL},
                   {TTL, TTL}
                  }

tab: 10,24 7

clock: dsn = { name = "DSN",
               edge = rising,
               channel = 77,
               qualifier = { channels = { 78 },
                             required = no,
                             levels = { high }
                           }
             }

clock: asn = { name = "ASN",
               edge = rising,
               channel = 76,
               qualifier = { channels = { 78 },
                             required = no,
                             levels = { high }
                           }
             }

label: FC2_0 =    { name = "FC2_0",
		 radix = bin,
                channels = { 66, 65, 64 }
               }

label: RWN =  { name = "R/WN",
		 radix = bin,
                 channels = { 67 }
               }

label: SZ =  { name = "SZ",
		 radix = bin,
                 channels = { 69, 68 }
               }

label: ADDRESS ={ name = "ADDRESS",
                channels = { 31, 30, 29, 28, 27, 26, 25, 24,
                	     23, 22, 21, 20, 19, 18, 17, 16,
			     15, 14, 13, 12, 11, 10,  9,  8,
			      7,  6,  5,  4,  3,  2,  1,  0}
               }

label: DATA =  { name = "DATA",
                parts = 2,
		type = big,
                channels = { 63, 62, 61, 60, 59, 58, 57, 56,
			     55, 54, 53, 52, 51, 50, 49, 48,
		             47, 46, 45, 44, 43, 42, 41, 40,
			     39, 38, 37, 36, 35, 34, 33, 32}
               }


clockseq: SEQ1 = { dsn }
clockseq: SEQ2 = { asn }


%% START
! STATUS_TAB !


%% STATUS_TAB
(FC2_0 = FC_PROGRAM, RWN=0b1, SZ=0b00)	! decode !
(FC2_0 = FC_DATA,    RWN=0b1)		! MR !	/* memory read */
(FC2_0 = FC_DATA,    RWN=0b0)		! MW !	/* memory write */
(FC2_0 = 0b...)				! "reserved function code" !


%% decode	/* _ in a condition is only used for readability */
(DATA = 0b0011_0110_0011_1100)	 ! "MOVE.W\t#" FindNextOpc PROG \
				   print[0] ",D3" !
(DATA = 0b0011_0010_0001_0010)	 ! "MOVE.W\t(A2),D1" FindDataRead MR !
(DATA = 0b0011_1010_1000_0001)	 ! "MOVE.W\tD1,(A5)" FindDataWrite MW !
(DATA = 0b0010_0110_0111_1100)	 ! "MOVEA.L\t#" FindNextOpc PROG \
				   print[0] FindNextOpc PROG \
				   print[0] ",A3"!
(DATA = 0b0101_[...]1_0100_0011) ! "SUBQ.W\t#%d,D3" !
(DATA = 0b0110_0001_...._....)	 ! "BSR\t" \
				   Compute8bitBrAddr print[1]  \
				   { last_opc = TELL; last_opc_part = TELLPART } \
				   FindNewOpcAddress[0] CheckBrAddr \
				   GOTO[last_opc] GOTOPART[last_opc_part] \
				   FindDataWrite MW !
(DATA = 0b0100_1110_0111_0101)	 ! "RTS\t" \
				   { last_opc = TELL; last_opc_part = TELLPART } \
				   FindDataRead \
				   ComputeReturnAddr[0] print[1] \
				   GOTO[last_opc] GOTOPART[last_opc_part] \
				   FindNewOpcAddress[0] CheckBrAddr !
(DATA = 0b0110_0111_...._....)	 ! "BEQ\t" Compute8bitBrAddr print[1] \
				   FindNewOpcAddress[0] CheckBrAddr !
(DATA = 0b0110_0110_...._....)	 ! "BNE\t" Compute8bitBrAddr print[1] \
				   FindNewOpcAddress[0] CheckBrAddr !

%% FindNextOpc
(ADDRESS=0b[<.>32], TELLPART = 0) ! { nopc_addr = $1; nopc_part = 1 } \
				    NEXTPART ChkNextOpc !
(ADDRESS=0b[<.>32], TELLPART = 1) ! { nopc_addr = $1 + 4; nopc_part = 0 } \
				    NEXTPART ChkNextOpc !


%% ChkNextOpc	/* lookup table */
(ADDRESS = nopc_addr, TELLPART = nopc_part, FC2_0 = FC_PROGRAM, \
 RWN = 0b1, SZ = 0b00) 		! /* found next opcode */ !
(FC2_0 != FC_PROGRAM)	        ! NEXT ChkNextOpc !
	/* if we didn't find the next opcode function-code FC_PROGRAM is illegal
	   The disassembler looses his synchronisation status */


%% FindDataRead
(FC2_0 = FC_DATA, RWN = 0b1, datatr_opcpipe > 2) ! /* found */ !
(FC2_0 = FC_PROGRAM)	! /* not found yet */ \
		           { datatr_opcpipe = datatr_opcpipe + 1 } \
		           NEXTPART FindDataRead !
(FC2_0 = 0b...)	 	! /* not found yet */ NEXT FindDataRead !

 
%% FindDataWrite
(FC2_0 = FC_DATA, RWN = 0b0, datatr_opcpipe > 2) ! /* found */ !
(FC2_0 = FC_PROGRAM)	! /* not found yet */ \
			  { datatr_opcpipe = datatr_opcpipe + 1 } \
			  NEXTPART FindDataWrite !
(FC2_0 = 0b...)	 	! /* not found yet */ NEXT FindDataWrite !


%% FindNewOpcAddress
! FindNextOpc { state = TELL; part = TELLPART } FindAddrSeqBrk !


%% FindAddrSeqBrk
(pipeline > MAXPIPE)		  ! /* not found, reposition */ \
				    GOTO[state] GOTOPART[part] !
(ADDRESS=0b[<.>32], TELLPART = 0) ! { nopc_addr = $1; nopc_part = 1 } \
				    { pipeline = pipeline + 1 } \
				    NEXTPART ChkAddrSeq !
(ADDRESS=0b[<.>32], TELLPART = 1) ! { nopc_addr = $1 + 4; nopc_part = 0 } \
				    { pipeline = pipeline + 1 } \
				    NEXTPART ChkAddrSeq !


%% ChkAddrSeq
(ADDRESS = nopc_addr, TELLPART = nopc_part, FC2_0 = FC_PROGRAM, \
 RWN = 0b1, SZ = 0b00) 		! FindAddrSeqBrk !
(FC2_0 != FC_PROGRAM)		! NEXT ChkAddrSeq !
(FC2_0 = FC_PROGRAM)		! /* found addr sequence break, ready */ !


%% print
(DATA=0x[....])	                 ! "%04x"     !
(dest_address = 0x[........])	 ! "\t{%08x}" !


%% Compute8bitBrAddr	/* data label contains 8-bit signed offset to ADDRESS */
(DATA=0b<.>8_0[<.>7], ADDRESS=0x[........], TELLPART = 0) ! /* positive offset */ \
		{dest_address = $2 + $1;     /* compute destination address */ \
		 dest_address = dest_address + 2 } /* adjust for ADDRESS value */ \
		 CompNewAddr /* adjust for ADDRESS bus */ \
		"+%d"                         /* print offset */ !
(DATA=0b<.>8_0[<.>7], ADDRESS=0x[........], TELLPART = 1) ! /* positive offset */ \
		{dest_address = $2 + $1;     /* compute destination address */ \
		 dest_address = dest_address + 4 } /* adjust for ADDRESS value */ \
		 CompNewAddr /* adjust for ADDRESS bus */ \
		"+%d"                         /* print offset */ !
(DATA=0b<.>8_1[<.>7], ADDRESS=0x[........], TELLPART = 0) ! /* negative offset */ \
		{$1 = $1 - 0x80; 	      /* sign extend $1 */ \
		 dest_address = $2 + $1;      /* compute destination address */ \
		 dest_address = dest_address + 2 } /* adjust for ADDRESS value */ \
		 CompNewAddr /* adjust for ADDRESS bus */ \
		"%d"                          /* print offset */ !
(DATA=0b<.>8_1[<.>7], ADDRESS=0x[........], TELLPART = 1) ! /* negative offset */ \
		{$1 = $1 - 0x80; 	      /* sign extend $1 */ \
		 dest_address = $2 + $1;      /* compute destination address */ \
		 dest_address = dest_address + 4 } /* adjust for ADDRESS value */ \
		 CompNewAddr /* adjust for ADDRESS bus */ \
		"%d"                          /* print offset */ !


%% ComputeReturnAddr
(DATA=0x[....])		! { dest_address = $1 << 16 } NEXTPART \
			  ComputeReturnAddr[1] !
(DATA=0x[....])		! { dest_address = dest_address + $1 } MR \
			  CompNewAddr /* adjust for ADDRESS bus */ !


%% CompNewAddr
(dest_address = 0b<.>30_10)	! { new_address = dest_address - 2; \
				new_part = 1 } !
(dest_address = 0b<.>30_00)	! { new_address = dest_address ; \
				new_part = 0 } !


%% CheckBrAddr
(ADDRESS = new_address /* new_address computed in CompNewAddr table */ ) \
				/* branch executed */ \
				! { new_state = TELL } \
				GOTO[state] GOTOPART[part] SelUnused !
(ADDRESS = 0x........)		! /* branch not executed */ !


%% SelUnused
(state < new_state, FC2_0 = FC_PROGRAM) \
				! UNUSED NEXTPART { state = TELL } SelUnused !
(state = new_state, TELLPART < new_part ) ! UNUSED NEXTPART SelUnused !
(state = new_state)		! /* ready */ !
(state = 0x....)		! NEXTPART { state = TELL} SelUnused !
