/*-----------------------------------------------------------*/
/*                                                           */
/*  Cdisa for disassembling the i2c-bus signals              */
/*                                                           */ 
/*  author: Rini van Dijk                                    */
/*  dep.  : development DTS - IE                             */
/*          Eindhoven                        date: 23-07-92  */
/*                                                           */
/*-----------------------------------------------------------*/
%%DEF

int  tmp, b10;

%%EQU

LT: START_STOP,ADDR_DATA, STRT, ADDR, DATA, ACK, L_10BADRW, L_10BADRR

%%FORMAT

logo: " I2C "
head: " S/P    ADDR  R/WN  DATA  ACK " 30

pods:threshold={{TTL,TTL},
                {TTL,TTL},
                {TTL,TTL}
               }

clock:ST_CLK  ={ name     ="ST_CLK",
                 edge     = any,
                 channel  = 15,
                 display  = state,
                 timing   = none,
                 polarity = +
               }

clock:D_CLK   ={ name     ="D_CLK",
                 edge     = falling,
                 channel  = 14,
                 display  = state,
                 timing   = none,
                 polarity = +
               }

label:DATA    ={ name     ="DATA",
                 channels = { 23,22,21,20,19,18,17,16 },
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               }

label:ACK     ={ name     ="ACK",
                 channels = {24},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               }

label:RWN     ={ name     ="RWN",
                 channels = {16},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               } 

label:DTADN   ={ name     ="DTADN",
                 channels = {30},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               } 

label:STSP    ={ name     ="STSP",
                 channels = {32},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {ST_CLK},
                 required = yes
               } 

label:SCL     ={ name     ="SCL",
                 channels = {9},
                 timing   = dataandtrigger,
                 display  = timing,
                 polarity = +,
                 required = no
               }

label:SDA     ={ name     ="SDA",
                 channels = {10},
                 timing   = dataandtrigger,
                 display  = timing,
                 polarity = +,
                 required = no
               }

clockseq:STRTSTP = {ST_CLK}
clockseq:DTA     = {D_CLK}

%%START

!STRT!

/*-----------------------------------------------------------*/
/*   defining start en stop conditions                       */
/*-----------------------------------------------------------*/
%%STRT
(STRTSTP)                ! START_STOP  !
(DTA)                    ! ADDR_DATA   !

%%START_STOP
( STSP = 0b1 )           ! " start  "   !
( STSP = 0b0 )           ! " stop   "   !

/*-----------------------------------------------------------*/
/*   test if data_byte is address or data                    */
/*-----------------------------------------------------------*/
%%ADDR_DATA
( DTADN = 0b0 )          !    ADDR      !
( DTADN = 0b1 )          !    DATA      !

/*-----------------------------------------------------------*/
/*   if address then test if read or write and display       */
/*   ADDR needs to be further decoded for 10bits addr and    */
/*   CBUS addresses                                          */
/*-----------------------------------------------------------*/
%%ADDR

( DATA = 0b[0000000]_0 )   ! "        general call address " !
( DATA = 0b[0000000]_1 )   ! "        START  byte          " !
( DATA = 0b[0000001]_. )   ! "        CBUS   address       " !
( DATA = 0b[0000010]_. )   ! "        reserved bus format  " !
( DATA = 0b[0000011]_. )   ! "        future purposes      " !
( DATA = 0b[00001..]_. )   ! "        future purposes      " !
( DATA = 0b[11111..]_. )   ! "        future purposes      " !
( DATA = 0b11110[..]0 )    !   {tmp =$1; tmp=tmp << 8} next  L_10BADRW ! 
( DATA = 0b11110[..]1 )    !   {tmp =$1; tmp=tmp << 8} next  L_10BADRR !

( DATA = 0b[<.>7]_0)       ! " \t   %02x   wr " !
( DATA = 0b[<.>7]_1)       ! " \t   %02x   rd " !

%%L_10BADRW
( DATA = 0x[..] )          !  { $1=$1+tmp } " \t  %03x    wr " !

%%L_10BADRR
( DATA = 0x[..] )          !  { $1=$1+tmp } " \t  %03x    rd " !

/*-----------------------------------------------------------*/
/*   if data then display data and acknowledge               */
/*-----------------------------------------------------------*/
%%DATA
( DATA = 0x[..])         ! " \t\t      %02x"  ACK !

%%ACK
( ACK  = 0b0 )           ! "    ack"  !
( ACK  = 0b1 )           ! "    nack" !
